@@ -52,10 +52,7 @@ epub_book = function(
     },
     post_processor = function(metadata, input, output, clean, verbose) {
       unlink(css)
-      if (is.null(opts$get('output_dir'))) return(output)
-      output2 = output_path(output)
-      file.rename(output, output2)
-      output2
+      move_output(output)
     }
   )
   config$bookdown_output_format = 'epub'
@@ -63,11 +60,19 @@ epub_book = function(
   config
 }
 
+move_output = function(output) {
+  if (is.null(opts$get('output_dir'))) return(output)
+  output2 = output_path(output)
+  file.rename(output, output2)
+  output2
+}
+
 process_markdown = function(input_file, from, pandoc_args, global) {
   intermediate_html = with_ext(input_file, 'tmp.html')
   on.exit(unlink(intermediate_html), add = TRUE)
   rmarkdown::pandoc_convert(
-    input_file, 'html', from, intermediate_html, TRUE, c(pandoc_args, '--section-divs')
+    input_file, 'html', from, intermediate_html, TRUE,
+    c(pandoc_args, '--section-divs', '--mathjax', '--number-sections')
   )
   x = readUTF8(intermediate_html)
   figs = parse_fig_labels(x, global)
@@ -75,6 +80,7 @@ process_markdown = function(input_file, from, pandoc_args, global) {
   content = resolve_refs_md(
     readUTF8(input_file), c(figs$ref_table, parse_section_labels(x))
   )
+  content = resolve_ref_links_epub(content)
   content = restore_part_epub(content)
   content = restore_appendix_epub(content)
   content = protect_math_env(content)
@@ -84,14 +90,19 @@ process_markdown = function(input_file, from, pandoc_args, global) {
 resolve_refs_md = function(content, ref_table) {
   ids = names(ref_table)
   # replace (\#fig:label) with Figure x.x:
-  for (i in grep('^(<p class="caption|<caption>|Table:)|(!\\[.*?\\]\\(.+?\\))', content)) {
+  for (i in grep('^(<p class="caption|<caption>|Table:|\\\\BeginKnitrBlock)|(!\\[.*?\\]\\(.+?\\))', content)) {
     for (j in ids) {
-      if (grepl(j, content[i], fixed = TRUE)) {
-        type = ifelse(grepl('^fig:', j), 'Figure', 'Table')
+      m = sprintf('\\(\\\\#%s\\)', j)
+      if (grepl(m, content[i])) {
+        id = ''; sep = ':'
+        type = gsub('^([^:]+).*$', '\\1', j)
+        if (type %in% names(theorem_abbr)) {
+          id = sprintf('<span id="%s"></span>', j)
+          sep = ''
+        }
+        label = label_prefix(type)
         content[i] = sub(
-          sprintf('\\(\\\\#%s\\)', j),
-          sprintf('<span id="%s"></span>%s %s: ', j, type, ref_table[j]),
-          content[i]
+          m, paste0(id, label, ref_table[j], ' '), content[i]
         )
         break
       }
@@ -99,6 +110,8 @@ resolve_refs_md = function(content, ref_table) {
   }
   # remove labels in figure alt text (it will contain \ like (\#fig:label))
   content = gsub('"\\(\\\\#(fig:[-[:alnum:]]+)\\)', '"', content)
+  # replace (\#eq:label) with equation numbers
+  content = add_eq_numbers(content, ids, ref_table)
 
   # look for \@ref(label) and resolve to actual figure/table/section numbers
   m = gregexpr('(?<!`)\\\\@ref\\(([-:[:alnum:]]+)\\)', content, perl = TRUE)
@@ -107,6 +120,39 @@ resolve_refs_md = function(content, ref_table) {
   content
 }
 
+# change labels (\#eq:label) in math environments into actual numbers in \tag{}
+add_eq_numbers = function(x, ids, ref_table) {
+  ids = grep('^eq:', ids, value = TRUE)
+  if (length(ids) == 0) return(x)
+  ref_table = ref_table[ids]
+  env = paste(math_envs, collapse = '|')
+  # no white spaces allowed after \begin|end{env}, and I added spaces for those
+  # env in verbatim chunks so so they won't be recognized and I can display
+  i1 = grep(sprintf('^\\\\begin\\{(%s)\\}$', env), x)
+  i2 = grep(sprintf('^\\\\end\\{(%s)\\}$', env), x)
+  if (length(i1) * length(i2) == 0) return(x)
+  i3 = unlist(mapply(seq, i1, next_nearest(i1, i2), SIMPLIFY = FALSE))
+  i3 = i3[grep('\\(\\\\(#eq:[-/[:alnum:]]+)\\)', x[i3])]
+  for (i in i3) {
+    for (j in ids) {
+      m = sprintf('\\(\\\\#%s\\)', j)
+      if (grepl(m, x[i])) {
+        # it is weird that \tag{} does not work in iBooks, so I have to cheat by
+        # using \qquad then the (equation number)
+        x[i] = sub(m, sprintf('\\\\qquad(%s)', ref_table[j]), x[i])
+        break
+      }
+    }
+  }
+  x
+}
+
+resolve_ref_links_epub = function(x) {
+  res = parse_ref_links(x, '^%s (.+[^ ])$')
+  if (is.null(res)) return(x)
+  restore_ref_links(res$content, '(?<!`)%s', res$tags, res$txts, TRUE)
+}
+
 reg_part = '^# \\(PART\\) .+ \\{-\\}$'
 
 # simply remove parts in epub
@@ -126,10 +172,13 @@ restore_appendix_epub = function(x) {
   x
 }
 
+# may add more LaTeX environments later
+math_envs = c('equation', 'align', 'eqnarray', 'gather')
+
 # wrap math environments in $$, otherwise they are discarded by Pandoc
 # https://github.com/jgm/pandoc/issues/2758
 protect_math_env = function(x) {
-  env = 'equation'  # may add more LaTeX environments later
+  env = c(math_envs, paste0(math_envs, '*'))
   s1 = sprintf('\\begin{%s}', env)
   s2 = sprintf('\\end{%s}', env)
   for (s in s1) {
